@@ -1,14 +1,19 @@
 import json
-from collections.abc import Mapping, MutableSequence, Sequence
+import typing
+from collections.abc import Iterable, Mapping, MutableMapping, MutableSequence, Sequence
+from copy import deepcopy
+from dataclasses import fields, is_dataclass
 from types import EllipsisType, GenericAlias
 from typing import (
     Any,
     ClassVar,
     Generic,
+    Literal,
     Self,
     TypeVar,
     cast,
     dataclass_transform,
+    overload,
 )
 from weakref import WeakValueDictionary
 
@@ -17,56 +22,67 @@ from haiway.attributes.annotations import (
     ObjectAttribute,
     resolve_self_attribute,
 )
-from haiway.attributes.coding import StateJSONEncoder
+from haiway.attributes.coding import AttributesJSONEncoder
 from haiway.attributes.path import AttributePath
+from haiway.attributes.specification import type_specification
 from haiway.attributes.validation import ValidationContext, ValidationError
-from haiway.types import MISSING, DefaultValue, Missing, not_missing
-from haiway.types.immutable import Immutable
+from haiway.types import (
+    MISSING,
+    DefaultValue,
+    Immutable,
+    Missing,
+    TypeSpecification,
+    not_missing,
+)
 
-__all__ = ("State",)
+__all__ = (
+    "Attribute",
+    "State",
+)
 
 
-class StateField[Value](Immutable):
-    """
-    Represents a field in a State class with its metadata.
-
-    This class holds information about a specific attribute of a State class,
-    including its name, typing, default value, and validation rules.
-    It is used internally by the State metaclass to manage state attributes
-    and ensure their immutability and type safety.
-    """
-
+class Attribute[Value](Immutable):
     name: str
+    alias: str | None
     annotation: AttributeAnnotation
+    required: bool
     default: DefaultValue[Value]
+    specification: TypeSpecification | None
 
     def validate(
         self,
-        value: Any | Missing,
+        value: Any,
         /,
     ) -> Value:
-        """
-        Validate and potentially transform the provided value.
-
-        If the value is MISSING, the default value is used instead.
-        The value (or default) is then passed through the validator.
-
-        Parameters
-        ----------
-        value : Any | Missing
-            The value to validate, or MISSING to use the default
-
-        Returns
-        -------
-        Value
-            The validated and potentially transformed value
-        """
         if value is MISSING:
             return self.annotation.validate(self.default())
 
         else:
             return self.annotation.validate(value)
 
+    def validate_from(
+        self,
+        mapping: Mapping[str, Any],
+        /,
+    ) -> Value:
+        value: Any
+        if self.alias is None:
+            value = mapping.get(
+                self.name,
+                self.default(),
+            )
+
+        else:
+            value = mapping.get(
+                self.alias,
+                mapping.get(
+                    self.name,
+                    self.default(),
+                ),
+            )
+
+        return self.annotation.validate(value)
+
 
 @dataclass_transform(
     kw_only_default=True,
@@ -89,6 +105,12 @@ class StateMeta(type):
     and validation logic.
     """
 
+    __IMMUTABLE__: EllipsisType = ...
+    __SELF_ATTRIBUTE__: ObjectAttribute
+    __TYPE_PARAMETERS__: Mapping[str, Any] | None
+    __SPECIFICATION__: TypeSpecification | None
+    __FIELDS__: Sequence[Attribute]
+
     def __new__(
         mcs,
         /,
@@ -105,25 +127,63 @@ class StateMeta(type):
             namespace,
             **kwargs,
         )
-
         self_attribute: ObjectAttribute = resolve_self_attribute(
             cls,
             parameters=type_parameters or {},
         )
-        fields: MutableSequence[StateField] = []
+        specification_fields: MutableMapping[str, TypeSpecification] | None = {}
+        required_fields: MutableSequence[str] = []
+        fields: MutableSequence[Attribute] = []
         for key, attribute in self_attribute.attributes.items():
             default: Any = getattr(cls, key, MISSING)
-            fields.append(
-                StateField(
-                    name=key,
-                    annotation=attribute,
-                    default=_resolve_default(default),
-                )
-            )
+            alias: str | None = attribute.alias
+            description: str | None = attribute.description
+            required: bool = attribute.required
 
-        cls.__SELF_ATTRIBUTE__ = self_attribute  # pyright: ignore[reportAttributeAccessIssue]
-        cls.__TYPE_PARAMETERS__ = type_parameters  # pyright: ignore[reportAttributeAccessIssue]
-        cls.__FIELDS__ = tuple(fields)  # pyright: ignore[reportAttributeAccessIssue]
+            specification: TypeSpecification | None = attribute.specification
+            if specification is None:
+                specification = type_specification(
+                    attribute,
+                    description=description,
+                )
+
+            field: Attribute = Attribute(
+                name=key,
+                alias=alias,
+                annotation=attribute,
+                required=required,
+                default=_resolve_default(default),
+                specification=specification,
+            )
+            fields.append(field)
+            if specification_fields is not None:
+                if specification is None:
+                    # there will be no specification at all
+                    specification_fields = None
+
+                elif field.alias is not None:
+                    specification_fields[field.alias] = specification
+                    if field.required:
+                        required_fields.append(field.alias)
+
+                else:
+                    specification_fields[field.name] = specification
+                    if field.required:
+                        required_fields.append(field.name)
+
+        cls.__SELF_ATTRIBUTE__ = self_attribute
+        cls.__TYPE_PARAMETERS__ = type_parameters
+        cls.__SPECIFICATION__ = (
+            {  # pyright: ignore[reportAttributeAccessIssue]
+                "type": "object",
+                "properties": specification_fields,
+                "required": required_fields,
+                "additionalProperties": False,
+            }
+            if specification_fields is not None
+            else None
+        )
+        cls.__FIELDS__ = tuple(fields)
         cls.__slots__ = tuple(field.name for field in fields)  # pyright: ignore[reportAttributeAccessIssue]
         cls.__match_args__ = cls.__slots__  # pyright: ignore[reportAttributeAccessIssue]
         cls._ = AttributePath(cls, attribute=cls)  # pyright: ignore[reportCallIssue, reportUnknownMemberType, reportAttributeAccessIssue]
@@ -219,19 +279,6 @@ class StateMeta(type):
 def _resolve_default[Value](
     value: DefaultValue[Value] | Value | Missing,
 ) -> DefaultValue[Value]:
-    """
-    Ensure a value is wrapped in a DefaultValue container.
-
-    Parameters
-    ----------
-    value : DefaultValue[Value] | Value | Missing
-        The value or default value container to resolve
-
-    Returns
-    -------
-    DefaultValue[Value]
-        The value wrapped in a DefaultValue container
-    """
     if isinstance(value, DefaultValue):
         return cast(DefaultValue[Value], value)
 
@@ -290,10 +337,6 @@ class State(metaclass=StateMeta):
     """
 
     _: ClassVar[Self]
-    __IMMUTABLE__: ClassVar[EllipsisType] = ...
-    __TYPE_PARAMETERS__: ClassVar[Mapping[str, Any] | None] = None
-    __SELF_ATTRIBUTE__: ClassVar[ObjectAttribute]
-    __FIELDS__: ClassVar[Sequence[StateField]]
 
     @classmethod
     def __class_getitem__(
@@ -314,12 +357,6 @@ class State(metaclass=StateMeta):
         -------
         type[Self]
             A specialized version of the class
-
-        Raises
-        ------
-        AssertionError
-            If the class is not generic or is already specialized,
-            or if the number of type arguments doesn't match the parameters
         """
         assert Generic in cls.__bases__, "Can't specialize non generic type!"  # nosec: B101
         assert cls.__TYPE_PARAMETERS__ is None, "Can't specialize already specialized type!"  # nosec: B101
@@ -399,15 +436,14 @@ class State(metaclass=StateMeta):
         TypeError
             If the value cannot be converted to an instance of this class
         """
-        match value:
-            case validated if isinstance(validated, cls):
-                return validated
+        if isinstance(value, cls):
+            return value
 
-            case {**values}:
-                return cls(**values)
+        elif isinstance(value, Mapping | typing.Mapping):
+            return cls(**value)
 
-            case _:
-                raise TypeError(f"'{value}' is not matching expected type of '{cls}'")
+        else:
+            raise TypeError(f"'{value}' is not matching expected type of '{cls}'")
 
     @classmethod
     def from_mapping(
@@ -415,8 +451,78 @@ class State(metaclass=StateMeta):
         value: Mapping[str, Any],
         /,
     ) -> Self:
+        """
+        Build an instance from a mapping of attribute values.
+
+        Parameters
+        ----------
+        value : Mapping[str, Any]
+            Mapping containing attribute names or aliases and their values.
+
+        Returns
+        -------
+        Self
+            New instance constructed from the provided mapping.
+        """
         return cls(**value)
 
+    @overload
+    @classmethod
+    def json_schema(
+        cls,
+        *,
+        indent: int | None = None,
+        required: Literal[True],
+    ) -> str: ...
+
+    @overload
+    @classmethod
+    def json_schema(
+        cls,
+        *,
+        indent: int | None = None,
+        required: Literal[False] = False,
+    ) -> str | None: ...
+
+    @classmethod
+    def json_schema(
+        cls,
+        *,
+        indent: int | None = None,
+        required: bool = False,
+    ) -> str | None:
+        """
+        Render this State's JSON Schema definition.
+
+        Parameters
+        ----------
+        indent : int | None, optional
+            Indentation passed to ``json.dumps`` for pretty-printing.
+        required : bool, default=False
+            When ``True``, raises if the class has no specification.
+
+        Returns
+        -------
+        str | None
+            JSON Schema string when available; ``None`` if no schema is defined
+            and ``required`` is ``False``.
+
+        Raises
+        ------
+        TypeError
+            If ``required`` is ``True`` but the class does not declare a schema.
+        """
+        if cls.__SPECIFICATION__ is not None:
+            return json.dumps(
+                cls.__SPECIFICATION__,
+                indent=indent,
+            )
+
+        elif required:
+            raise TypeError(f"{cls.__name__} cannot be represented using json schema")
+
+        return None
+
     @classmethod
     def from_json(
         cls,
@@ -424,6 +530,26 @@ class State(metaclass=StateMeta):
         /,
         decoder: type[json.JSONDecoder] = json.JSONDecoder,
     ) -> Self:
+        """
+        Deserialize an instance from a JSON object payload.
+
+        Parameters
+        ----------
+        value : str | bytes
+            JSON payload representing a single instance.
+        decoder : type[json.JSONDecoder], default=json.JSONDecoder
+            Decoder class used by ``json.loads``.
+
+        Returns
+        -------
+        Self
+            Instance built from the decoded payload.
+
+        Raises
+        ------
+        ValueError
+            If the payload cannot be decoded or does not match the schema.
+        """
         try:
             return cls(
                 **json.loads(
@@ -442,6 +568,26 @@ class State(metaclass=StateMeta):
         /,
         decoder: type[json.JSONDecoder] = json.JSONDecoder,
     ) -> Sequence[Self]:
+        """
+        Deserialize a sequence of instances from a JSON array payload.
+
+        Parameters
+        ----------
+        value : str | bytes
+            JSON payload representing an array of objects.
+        decoder : type[json.JSONDecoder], default=json.JSONDecoder
+            Decoder class used by ``json.loads``.
+
+        Returns
+        -------
+        Sequence[Self]
+            Tuple of instances decoded from the array payload.
+
+        Raises
+        ------
+        ValueError
+            If decoding fails or the payload is not an array of valid objects.
+        """
         payload: Any
         try:
             payload = json.loads(
@@ -468,9 +614,29 @@ class State(metaclass=StateMeta):
     def to_json(
         self,
         indent: int | None = None,
-        encoder_class: type[json.JSONEncoder] = StateJSONEncoder,
+        encoder_class: type[json.JSONEncoder] = AttributesJSONEncoder,
     ) -> str:
-        mapping: Mapping[str, Any] = self.to_mapping()
+        """
+        Serialize this instance to a JSON string.
+
+        Parameters
+        ----------
+        indent : int | None, optional
+            Indentation passed to ``json.dumps`` for pretty-printing.
+        encoder_class : type[json.JSONEncoder], default=StateJSONEncoder
+            Encoder class responsible for encoding custom types.
+
+        Returns
+        -------
+        str
+            JSON representation of this instance.
+
+        Raises
+        ------
+        ValueError
+            If encoding fails.
+        """
+        mapping: Mapping[str, Any] = self.to_mapping(recursive=True)
         try:
             return json.dumps(
                 mapping,
@@ -509,12 +675,7 @@ class State(metaclass=StateMeta):
                 object.__setattr__(
                     self,  # pyright: ignore[reportUnknownArgumentType]
                     field.name,
-                    field.validate(
-                        kwargs.get(
-                            field.name,
-                            MISSING,
-                        )
-                    ),
+                    field.validate_from(kwargs),
                 )
 
     def updated(
@@ -552,15 +713,15 @@ class State(metaclass=StateMeta):
 
     def to_mapping(
         self,
-        recursive: bool = False,
+        recursive: bool = True,
     ) -> Mapping[str, Any]:
         """
         Convert this instance to a mapping of attribute names to values.
 
         Parameters
         ----------
-        recursive : bool, default=False
-            If True, nested State instances are also converted to mappings
+        recursive : bool, default=True
+            If True, nested instances are also converted to mappings
 
         Returns
         -------
@@ -568,13 +729,20 @@ class State(metaclass=StateMeta):
             A mapping of attribute names to values
         """
         dict_result: dict[str, Any] = {}
-        for field in self.__FIELDS__:
-            value: Any | Missing = getattr(self, field.name, MISSING)
-            if recursive and isinstance(value, State):
-                dict_result[field.name] = value.to_mapping(recursive=recursive)
+        if recursive:
+            for field in self.__FIELDS__:
+                key: str = field.alias if field.alias is not None else field.name
+                value: Any | Missing = getattr(self, field.name, MISSING)
 
-            elif not_missing(value):
-                dict_result[field.name] = value
+                if not_missing(value):
+                    dict_result[key] = _recursive_mapping(value)
+
+        else:
+            for field in self.__FIELDS__:
+                key: str = field.alias if field.alias is not None else field.name
+                value: Any | Missing = getattr(self, field.name, MISSING)
+                if not_missing(value):
+                    dict_result[key] = value
 
         return dict_result
 
@@ -587,11 +755,25 @@ class State(metaclass=StateMeta):
         str
             A string representation in the format "ClassName(attr1: value1, attr2: value2)"
         """
-        attributes: str = ", ".join([f"{key}: {value}" for key, value in vars(self).items()])
+        attributes: str = ", ".join(
+            [
+                f"{field.alias or field.name}: {getattr(self, field.name)}"
+                for field in self.__class__.__FIELDS__
+            ]
+        )
         return f"{self.__class__.__name__}({attributes})"
 
     def __repr__(self) -> str:
-        return str(self)
+        """
+        Return the canonical representation of this instance.
+
+        Returns
+        -------
+        str
+            ``repr`` string mirroring ``__str__`` for readability.
+        """
+        attributes: str = ", ".join([f"{key}: {value}" for key, value in vars(self).items()])
+        return f"{self.__class__.__name__}({attributes})"
 
     def __eq__(
         self,
@@ -622,6 +804,14 @@ class State(metaclass=StateMeta):
         )
 
     def __hash__(self) -> int:
+        """
+        Compute a hash value for this immutable instance.
+
+        Returns
+        -------
+        int
+            Hash derived from non-missing attribute values.
+        """
         hash_values: list[int] = []
         for field in self.__FIELDS__:
             value: Any = getattr(self, field.name, MISSING)
@@ -644,6 +834,21 @@ class State(metaclass=StateMeta):
         name: str,
         value: Any,
     ) -> Any:
+        """
+        Disallow attribute assignment to preserve immutability.
+
+        Parameters
+        ----------
+        name : str
+            Attribute name being set.
+        value : Any
+            Incoming value (unused).
+
+        Raises
+        ------
+        AttributeError
+            Always raised to signal immutability.
+        """
         raise AttributeError(
             f"Can't modify immutable state {self.__class__.__qualname__},"
             f" attribute - '{name}' cannot be modified"
@@ -653,6 +858,19 @@ class State(metaclass=StateMeta):
         self,
         name: str,
     ) -> None:
+        """
+        Disallow attribute deletion to preserve immutability.
+
+        Parameters
+        ----------
+        name : str
+            Attribute name being deleted.
+
+        Raises
+        ------
+        AttributeError
+            Always raised to signal immutability.
+        """
         raise AttributeError(
             f"Can't modify immutable state {self.__class__.__qualname__},"
             f" attribute - '{name}' cannot be deleted"
@@ -712,12 +930,29 @@ class State(metaclass=StateMeta):
         Self
             A new instance with replaced values
         """
-        if not kwargs or kwargs.keys().isdisjoint(getattr(self, "__slots__", ())):
+        if not kwargs:
             return self  # do not make a copy when nothing will be updated
 
+        fields: Sequence[Attribute] = self.__class__.__FIELDS__
+        alias_to_name: dict[str, str] = {
+            field.alias if field.alias is not None else field.name: field.name for field in fields
+        }
+        valid_keys: set[str] = set(alias_to_name.keys()) | set(alias_to_name.values())
+
+        if kwargs.keys().isdisjoint(valid_keys):
+            return self  # do not make a copy when nothing will be updated
+
+        canonical_updates: dict[str, Any] = {}
+        for key, value in kwargs.items():
+            if key in valid_keys:
+                canonical_updates[alias_to_name.get(key, key)] = value
+
+        if not canonical_updates:
+            return self
+
         updated: Self = object.__new__(self.__class__)
-        for field in self.__class__.__FIELDS__:
-            update: Any | Missing = kwargs.get(field.name, MISSING)
+        for field in fields:
+            update: Any | Missing = canonical_updates.get(field.name, MISSING)
             if update is MISSING:  # reuse missing elements
                 object.__setattr__(
                     updated,
@@ -734,3 +969,30 @@ class State(metaclass=StateMeta):
                     )
 
         return updated
+
+
+def _recursive_mapping(  # noqa: PLR0911
+    value: Any,
+) -> Any:
+    if isinstance(value, str | bytes | float | int | bool | None):
+        return value
+
+    elif isinstance(value, State):
+        return value.to_mapping(recursive=True)
+
+    elif is_dataclass(value):
+        return {
+            field.name: _recursive_mapping(getattr(value, field.name)) for field in fields(value)
+        }
+
+    elif isinstance(value, Mapping | typing.Mapping):
+        return {key: _recursive_mapping(element) for key, element in value.items()}
+
+    elif isinstance(value, Iterable | typing.Iterable):
+        return [_recursive_mapping(element) for element in value]
+
+    elif hasattr(value, "to_mapping") and callable(value.to_mapping):
+        return value.to_mapping()
+
+    else:
+        return deepcopy(value)
