@@ -11,8 +11,7 @@ from typing import Any, ClassVar, Literal, Protocol, Self, runtime_checkable
 from uuid import UUID, uuid4
 
 from haiway.context.identifier import ScopeIdentifier
-from haiway.state import Immutable
-from haiway.types import Missing
+from haiway.types import Immutable, Missing
 from haiway.utils.formatting import format_str
 
 __all__ = (
@@ -216,7 +215,7 @@ class Observability(Immutable):  # avoiding State inheritance to prevent propaga
     scope_exiting: ObservabilityScopeExiting
 
 
-def _logger_observability(
+def _logger_observability(  # noqa: C901
     logger: Logger,
     /,
 ) -> Observability:
@@ -332,10 +331,17 @@ def _logger_observability(
     ) -> None:
         logger.log(
             ObservabilityLevel.DEBUG,
-            f"{scope.unique_name} Exiting scope: {scope.name}",
+            f"[{trace_id_hex}] {scope.unique_name} Exiting scope: {scope.name}",
             exc_info=exception,
         )
 
+        if isinstance(exception, Exception):
+            logger.log(
+                ObservabilityLevel.ERROR,
+                f"[{trace_id_hex}] {scope.unique_name} Scope error: {exception}",
+                exc_info=exception,
+            )
+
     return Observability(
         trace_identifying=trace_identifying,
         log_recording=log_recording,
@@ -393,15 +399,14 @@ class ObservabilityContext(Immutable):
 
         except LookupError:
             resolved_observability: Observability
-            match observability:
-                case Observability() as observability:
-                    resolved_observability = observability
+            if observability is None:
+                resolved_observability = _logger_observability(getLogger(scope.name))
 
-                case None:
-                    resolved_observability = _logger_observability(getLogger(scope.name))
+            elif isinstance(observability, Logger):
+                resolved_observability = _logger_observability(observability)
 
-                case Logger() as logger:
-                    resolved_observability = _logger_observability(logger)
+            else:
+                resolved_observability = observability
 
             # create root scope when missing
             return cls(
@@ -411,15 +416,14 @@ class ObservabilityContext(Immutable):
 
         # create nested scope otherwise
         resolved_observability: Observability
-        match observability:
-            case None:
-                resolved_observability = current.observability
+        if observability is None:
+            resolved_observability = current.observability
 
-            case Logger() as logger:
-                resolved_observability = _logger_observability(logger)
+        elif isinstance(observability, Logger):
+            resolved_observability = _logger_observability(observability)
 
-            case observability:
-                resolved_observability = observability
+        else:
+            resolved_observability = observability
 
         return cls(
             scope=scope,
@@ -628,12 +632,11 @@ class ObservabilityContext(Immutable):
         try:  # catch exceptions - we don't wan't to blow up on observability
             context: Self = cls._context.get()
 
-            if context.observability is not None:
-                context.observability.attributes_recording(
-                    context._scope,
-                    level=level,
-                    attributes=attributes,
-                )
+            context.observability.attributes_recording(
+                context._scope,
+                level=level,
+                attributes=attributes,
+            )
 
         except Exception as exc:
             cls.record_log(
@@ -672,11 +675,6 @@ class ObservabilityContext(Immutable):
         Enter this observability context.
 
         Sets this context as the current one and records scope entry.
-
-        Raises
-        ------
-        AssertionError
-            If attempting to re-enter an already active context
         """
         assert self._token is None, "Context reentrance is not allowed"  # nosec: B101
         object.__setattr__(
@@ -705,11 +703,6 @@ class ObservabilityContext(Immutable):
             Exception instance that caused the exit
         exc_tb: TracebackType | None
             Traceback for the exception
-
-        Raises
-        ------
-        AssertionError
-            If the context is not active
         """
         assert self._token is not None, "Unbalanced context enter/exit"  # nosec: B101
         ObservabilityContext._context.reset(self._token)  # pyright: ignore[reportArgumentType]
