@@ -1,10 +1,11 @@
-from asyncio import Task, TaskGroup, get_event_loop
+from asyncio import Task, TaskGroup, get_running_loop
 from collections.abc import Callable, Coroutine
 from contextvars import ContextVar, Token, copy_context
 from types import TracebackType
 from typing import Any, ClassVar
 
-from haiway.state import Immutable
+from haiway.context.variables import VariablesContext
+from haiway.types import Immutable
 
 __all__ = ("TaskGroupContext",)
 
@@ -23,7 +24,7 @@ class TaskGroupContext(Immutable):
     @classmethod
     def run[Result, **Arguments](
         cls,
-        function: Callable[Arguments, Coroutine[Any, Any, Result]],
+        coro: Callable[Arguments, Coroutine[Any, Any, Result]] | Coroutine[Any, Any, Result],
         /,
         *args: Arguments.args,
         **kwargs: Arguments.kwargs,
@@ -32,7 +33,7 @@ class TaskGroupContext(Immutable):
         Run a coroutine function as a task within the current task group.
 
         If called within a TaskGroupContext, creates a task in that group.
-        If called outside any TaskGroupContext, creates a detached task.
+        If called outside any TaskGroupContext, creates a background task.
 
         Parameters
         ----------
@@ -49,74 +50,70 @@ class TaskGroupContext(Immutable):
             The created task
         """
         try:
-            return cls._context.get().create_task(
-                function(*args, **kwargs),
-                context=copy_context(),
+            with VariablesContext(isolated=True):  # isolate variables
+                return cls._context.get().create_task(
+                    coro if isinstance(coro, Coroutine) else coro(*args, **kwargs),
+                    context=copy_context(),
+                )
+
+        except LookupError:  # spawn task in the background as a fallback
+            return cls.background_run(
+                coro,
+                *args,
+                **kwargs,
             )
 
-        except LookupError:  # spawn task out of group as a fallback
-            return get_event_loop().create_task(
-                function(*args, **kwargs),
-                context=copy_context(),
-            )
-
-    _group: TaskGroup
-    _token: Token[TaskGroup] | None = None
-
-    def __init__(
-        self,
-        task_group: TaskGroup | None = None,
-    ) -> None:
+    @classmethod
+    def background_run[Result, **Arguments](
+        cls,
+        coro: Callable[Arguments, Coroutine[Any, Any, Result]] | Coroutine[Any, Any, Result],
+        /,
+        *args: Arguments.args,
+        **kwargs: Arguments.kwargs,
+    ) -> Task[Result]:
         """
-        Initialize a task group context.
+        Run a coroutine function as a background detached task.
 
         Parameters
         ----------
-        task_group: TaskGroup | None
-            The task group to use, or None to create a new one
+        function: Callable[Arguments, Coroutine[Any, Any, Result]] | Coroutine[Any, Any, Result]
+            The coroutine function to run
+        *args: Arguments.args
+            Positional arguments to pass to the function
+        **kwargs: Arguments.kwargs
+            Keyword arguments to pass to the function
+
+        Returns
+        -------
+        Task[Result]
+            The created task
         """
-        object.__setattr__(
-            self,
-            "_group",
-            task_group if task_group is not None else TaskGroup(),
-        )
-        object.__setattr__(
-            self,
-            "_token",
-            None,
-        )
 
-    def __setattr__(
-        self,
-        name: str,
-        value: Any,
-    ) -> Any:
-        raise AttributeError(
-            f"Can't modify immutable {self.__class__.__qualname__},"
-            f" attribute - '{name}' cannot be modified"
-        )
+        with VariablesContext(isolated=True):  # isolate variables
+            # TODO: isolate further with detached state and observability scope?
+            # TODO: manage custom background task group?
+            return get_running_loop().create_task(
+                coro if isinstance(coro, Coroutine) else coro(*args, **kwargs),
+                context=copy_context(),
+            )
 
-    def __delattr__(
-        self,
-        name: str,
-    ) -> None:
-        raise AttributeError(
-            f"Can't modify immutable {self.__class__.__qualname__},"
-            f" attribute - '{name}' cannot be deleted"
-        )
+    _group: TaskGroup | None = None
+    _token: Token[TaskGroup] | None = None
 
     async def __aenter__(self) -> None:
         """
         Enter this task group context.
 
         Enters the underlying task group and sets this context as current.
-
-        Raises
-        ------
-        AssertionError
-            If attempting to re-enter an already active context
         """
         assert self._token is None, "Context reentrance is not allowed"  # nosec: B101
+        object.__setattr__(
+            self,
+            "_group",
+            TaskGroup(),
+        )
+
+        assert self._group is not None  # nosec: B101
         await self._group.__aenter__()
         object.__setattr__(
             self,
@@ -144,26 +141,29 @@ class TaskGroupContext(Immutable):
             Exception instance that caused the exit
         exc_tb: TracebackType | None
             Traceback for the exception
-
-        Raises
-        ------
-        AssertionError
-            If the context is not active
         """
         assert self._token is not None, "Unbalanced context enter/exit"  # nosec: B101
-        TaskGroupContext._context.reset(self._token)
-        object.__setattr__(
-            self,
-            "_token",
-            None,
-        )
+        assert self._group is not None  # nosec: B101
 
         try:
+            TaskGroupContext._context.reset(self._token)
             await self._group.__aexit__(
-                et=exc_type,
-                exc=exc_val,
-                tb=exc_tb,
+                exc_type,
+                exc_val,
+                exc_tb,
             )
 
-        except BaseException:
-            pass  # silence TaskGroup exceptions, if there was exception already we will get it
+        except ExceptionGroup:
+            pass  # skip task group exceptions
+
+        finally:
+            object.__setattr__(
+                self,
+                "_token",
+                None,
+            )
+            object.__setattr__(
+                self,
+                "_group",
+                None,
+            )
