@@ -1,18 +1,25 @@
+from haiway.types.alias import Alias
 from haiway.types.basic import BasicValue, RawValue
 from haiway.types.default import Default, DefaultValue
+from haiway.types.description import Description
 from haiway.types.immutable import Immutable
 from haiway.types.map import Map
 from haiway.types.missing import MISSING, Missing, is_missing, not_missing, unwrap_missing
+from haiway.types.specification import Specification, TypeSpecification
 
 __all__ = (
     "MISSING",
+    "Alias",
     "BasicValue",
     "Default",
     "DefaultValue",
+    "Description",
     "Immutable",
     "Map",
     "Missing",
     "RawValue",
+    "Specification",
+    "TypeSpecification",
     "is_missing",
     "not_missing",
     "unwrap_missing",
